module standard_tools;

import tool;
import icanvas;
import tk.types;
import tk.events;
import tk.geometry;
import std.math;

final class PanTool : Tool {
    bool handle_button_press(ICanvas canvas, in ButtonEvent event) {
        if (event.button_name == ButtonName.MIDDLE) {
            mLastPosition = event.screen_point;
            return true;
        }
        else {
            return false;
        }
    }

    bool handle_button_release(ICanvas canvas, in ButtonEvent event) {
        return true;
    }

    bool handle_motion(ICanvas canvas, in MotionEvent event) {
        if (event.mask.is_set(Modifier.BUTTON_MIDDLE)) {
            canvas.rel_pan(mLastPosition - event.screen_point);
            mLastPosition = event.screen_point;

            return true;
        }
        else {
            return false;
        }
    }

    bool handle_scroll(ICanvas canvas, in ScrollEvent event) {
        if (event.mask.is_unset(Modifier.BUTTON_MIDDLE)) {
            Vector delta;

            switch (event.scroll_direction) {
            case ScrollDirection.UP:
                delta = event.mask.is_set(Modifier.SHIFT) ? Vector(-AMOUNT, 0.0) : Vector(0.0, -AMOUNT);
                break;
            case ScrollDirection.DOWN:
                delta = event.mask.is_set(Modifier.SHIFT) ? Vector(AMOUNT, 0.0) : Vector(0.0, AMOUNT);
                break;
            case ScrollDirection.LEFT:
                delta = Vector(-AMOUNT, 0.0);
                break;
            case ScrollDirection.RIGHT:
                delta = Vector(AMOUNT, 0.0);
                break;
            }

            canvas.rel_pan(delta);
        }

        return true;
    }

    bool handle_key_press(ICanvas canvas, in KeyEvent event) {
        return false;
    }

    bool handle_key_release(ICanvas canvas, in KeyEvent event) {
        return false;
    }

    private {
        Point mLastPosition;
        static invariant double AMOUNT = 60.0;
    }
}

final class ZoomTool : Tool {
    bool handle_button_press(ICanvas canvas, in ButtonEvent event) {
        return false;
    }

    bool handle_button_release(ICanvas canvas, in ButtonEvent event) {
        return false;
    }

    bool handle_motion(ICanvas canvas, in MotionEvent event) {
        return false;
    }

    bool handle_scroll(ICanvas canvas, in ScrollEvent event) {
        if (event.mask.is_set(Modifier.CONTROL)) {
            if (event.scroll_direction == ScrollDirection.DOWN) {
                canvas.rel_zoom(event.screen_point(), 1.0 / ZOOM);
                return true;
            }
            else if (event.scroll_direction == ScrollDirection.UP) {
                canvas.rel_zoom(event.screen_point(), ZOOM);
                return true;
            }
            else {
                return false;
            }
        }
        else {
            return false;
        }
    }

    bool handle_key_press(ICanvas canvas, in KeyEvent event) {
        return false;
    }

    bool handle_key_release(ICanvas canvas, in KeyEvent event) {
        return false;
    }

    private {
        static invariant double ZOOM = sqrt(2.0);
    }
}
