module tool_stack;

import std.stdio;

import tool;
private import icanvas;
private import tk.types;
private import tk.geometry;
private import tk.events;

class ToolStack : ICanvasEventHandler {
    this(in Tool[] tools) {
        mTools = tools.dup;
    }

    override bool handle_button_press(ICanvas canvas, in ButtonEvent event) {
        writefln("%s", event);

        if (mGrabbedTool is null) {
            foreach_reverse(ref tool; mTools) {
                if (tool.handle_button_press(canvas, event)) {
                    mGrabbedTool = &tool;
                    mGrabbedButton = event.button_name;
                    break;
                }
            }
        }
        else {
            mGrabbedTool.handle_button_press(canvas, event);
        }

        return true;
    }

    override bool handle_button_release(ICanvas canvas, in ButtonEvent event) {
        writefln("%s", event);

        if (mGrabbedTool !is null) {
            mGrabbedTool.handle_button_release(canvas, event);

            if (mGrabbedButton == event.button_name) {
                mGrabbedTool = null;
            }
        }

        return true;
    }

    override bool handle_key_press(ICanvas canvas, in KeyEvent event) {
        writefln("%s", event);
        return true;
    }

    override bool handle_key_release(ICanvas canvas, in KeyEvent event) {
        writefln("%s", event);
        return true;
    }

    override bool handle_motion(ICanvas canvas, in MotionEvent event) {
        writefln("%s", event);

        if (mGrabbedTool is null) {
            foreach_reverse(ref tool; mTools) {
                if (tool.handle_motion(canvas, event)) {
                    break;
                }
            }
        }
        else {
            mGrabbedTool.handle_motion(canvas, event);
        }

        return true;
    }

    override bool handle_scroll(ICanvas canvas, in ScrollEvent event) {
        writefln("%s", event);

        if (mGrabbedTool is null) {
            foreach_reverse(ref tool; mTools) {
                if (tool.handle_scroll(canvas, event)) {
                    break;
                }
            }
        }
        else {
            mGrabbedTool.handle_scroll(canvas, event);
        }

        return true;
    }

    /*
    override void push(Tool tool) {
    }

    override void pop() {
    }

    override void replace(Tool tool) {
    }
    */

    private {
        Tool[] mTools;
        Tool * mGrabbedTool;
        ButtonName mGrabbedButton;
    }
}
